/***********************************************************************/
/*                                                                     */
/*  FILE        :ICS_sample.c                                          */
/*  DATE        :28, August, 2017                                       */
/*  DESCRIPTION :Main Program                                          */
/*  CPU TYPE    :RX63T                                                 */
/*                                                                     */
/*  NOTE:THIS IS A TYPICAL EXAMPLE.                                    */
/*                                                                     */
/***********************************************************************/

#ifdef  __ICCRX__
  #include "r_cg_macrodriver.h"
  #include <math.h>
#elif   __CCRX__
  #include <machine.h>
  #include <mathf.h>
  #include <stdint.h>
  #include "iodefine.h"
#else
  //
#endif

#include "ICS2_RX63T.h"

void main(void);

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Set option bytes */

#ifdef   __ICCRX__

  #if __LITTLE_ENDIAN__
    #pragma public_equ="__MDES",0xFFFFFFFFUL
  #else
    #pragma public_equ="__MDES",0xFFFFFFF8UL
  #endif

  #pragma public_equ="__OFS0",0xFFFFFFFFUL
  #pragma public_equ="__OFS1",0xFFFFFFFFUL

#elif   __CCRX__  
  
  
#endif


#define   TWOPI    (2.0f*3.1415926535f)
#define   BRR    0
#define   MODE   3     // 0:16bit 8ch,one time
                       // 1:32bit 8ch,2 times
                       // 2:32bit 4ch,one time
                       // 3:32bit 12ch 3 times

#ifdef  __ICCRX__
  #pragma location=0x03000
  __no_init  uint32_t dtc_table[256]; /* alignment 0x000 */
#elif   __CCRX__
  #pragma address dtc_table=0x03000
  uint32_t dtc_table[256];            /* alignment 0x000 */
#endif

//   CLK=160MHz/ 80MHz SPEED 0    2.500Mbps => ICS Clock 20MHz
//   CLK=160MHz/ 80MHz SPEED 1    1.250Mbps => ICS Clock 10MHz
//   CLK=192MHz/ 96MHz SPEED 0    3.000Mbps => ICS Clock 24MHz
//   CLK=192MHz/ 96MHz SPEED 1    1.500Mbps => ICS Clock 12MHz
//   CLK=192MHz/ 96MHz SPEED 2    1.000Mbps => ICS Clock  8MHz
//   CLK=200MHz/100MHz SPEED 0    3.125Mbps => ICS Clock 25MHz
//   CLK=200MHz/100MHz SPEED 1    1.625Mbps => ICS Clock 12.5MHz
//   CLK=200MHz/100MHz SPEED 2    1.042Mbps => ICS Clock 8.33MHz
//   CLK=240MHz/120MHz SPEED 0    3.750Mbps => ICS Clock 30MHz

  
  
float   wt    = 0.0f;
float   dwt   = TWOPI*50.0f/10000.0f;
float   refu  = 0.0f;
float   refv  = 0.0f;
float   refw  = 0.0f;
float   m     = 1.0f;
float   dm    = 0.0001;
int32_t up    = 0;


void main(void)
{
    SYSTEM.PRCR.WORD = 0xa50b;
    SYSTEM.OSTDCR.BYTE = 0x00;

    while(SYSTEM.MOSCCR.BYTE != 0x00)
    {
        SYSTEM.MOSCCR.BYTE = 0x00;
    }
//  SYSTEM.PLLCR.BIT.STC    7=x8, 9=x10, 11=x12, 15=x16, 19=x20, 23=x24, 24=x25, 49=x50  
//------------------------------------------------------------------------------------------
//  SYSTEM.PLLCR.BIT.STC   = 0x17;    //  8MHz*24=192MHz  RSKRX63T-H   (R5F563TEADFB 144pin) 
//  SYSTEM.PLLCR.BIT.STC   = 0x13;    // 10MHz*20=200MHz  AP-RX63TL-0A (R5F563T6ExFM  64pin)
//  SYSTEM.PLLCR.BIT.STC   = 0x0F;    // 12MHz*16=192MHz  RX63T-ExtBUS (R5F563TEBDFB 144pin)
    SYSTEM.PLLCR.BIT.STC   = 0x0F;    // 12MHz*16=192MHz  T6209 RX63TE (R5F563TEBDFB 144pin)
                                      // 10MHz*16=160MHz
   
    SYSTEM.SCKCR.LONG  = 0x21021222;  /*                  */
    SYSTEM.SCKCR3.WORD = 0x0400;

  //******************************************************************************************************
    ics2_init((void *)(&dtc_table), ICS_SCI0_PB2_PB1, 6, BRR, MODE); // T6209 CN11 (144,120,112,100,64,48pin)
  //ics2_init((void *)(&dtc_table), ICS_SCI0_P30_P24, 6, BRR, MODE); //            (64,48pin)
  //ics2_init((void *)(&dtc_table), ICS_SCI0_PA4_PA5, 6, BRR, MODE); // T6209 CN10 (144,120,112,100pin)
  //ics2_init((void *)(&dtc_table), ICS_SCI0_P23_P22, 6, BRR, MODE); // T6209 CN2  (144,120,112,100pin)
  
  //ics2_init((void *)(&dtc_table), ICS_SCI1_PD3_PD5, 6, BRR, MODE); // ---------- (144,120,112,100,64,48pin,JTAG)
  //ics2_init((void *)(&dtc_table), ICS_SCI1_P94_P93, 6, BRR, MODE); //            (64pin)
  //ics2_init((void *)(&dtc_table), ICS_SCI1_PF3_PF2, 6, BRR, MODE); // T6209 CN14 (144,120,112pin)
  //ics2_init((void *)(&dtc_table), ICS_SCI1_P95_P96, 6, BRR, MODE); // ---------- (144,120,112,100pin,INV)
  
  //ics2_init((void *)(&dtc_table), ICS_SCI2_P02_P03, 6, BRR, MODE); // T6209 CN4  (144pin)
  //ics2_init((void *)(&dtc_table), ICS_SCI2_PG0_PG1, 6, BRR, MODE); // T6209 CN12 (144,120,112pin)
  //ics2_init((void *)(&dtc_table), ICS_SCI2_PA1_PA2, 6, BRR, MODE); // T6209 CN8  (144,120,112,100pin)

  //ics2_init((void *)(&dtc_table), ICS_SCI3_P35_P34, 6, BRR, MODE); // T6209 CN1  (144pin)
  //ics2_init((void *)(&dtc_table), ICS_SCI3_PG3_PG4, 6, BRR, MODE); // T6209 CN13 (144pin)
 
  //ics2_init((void *)(&dtc_table), ICS_SCI12_PB5_PB6_100PorABOVE, 6, BRR, MODE); // T6209 CN9 (144,120,112,100pin)
  //ics2_init((void *)(&dtc_table), ICS_SCI12_P81_P80,             6, BRR, MODE); // T6209 CN3 (144,120,112,100pin)
  //*******************************************************************************************************

#ifdef  __ICCRX__
    EI();
#elif   __CCRX__
    set_ipl(0); /* set lowest IPL level */
    setpsw_i();  // enable interrput
#else
    //
#endif

    while (1)
    {
        int32_t i;
        
        wt = wt + dwt;
        if (wt>TWOPI) { wt = wt - TWOPI; }
        
        if (up==1)
        {
             m = m + dm;
             if (m>=1.0f) { up = 0; }
        }
        else
        {
             m = m - dm;
             if (m<=0.5f) { up = 1; }
        }

        refu  = m * sinf(wt);
        refv  = m * sinf(wt-TWOPI*1.0f/3.0f);
        refw  = m * sinf(wt-TWOPI*2.0f/3.0f);

        for(i=0;i<5000;i++) { nop(); }

        ics2_watchpoint();
    }
}
